

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import shutil
import tempfile
import yum

def _write_config (fd, repo_url):
    f = os.fdopen (fd, "w")
        
    f.write ("""
[main]

[repoke_tmp_repo]
name=Repoke Temporary Repository
baseurl=%s
""" % repo_url)

    f.close ()

def _get_requires (repo, po):
    requires = []
    for req in po.returnPrco ('requires'):
        (r, f, v) = req
        if r.startswith ('rpmlib('):
            continue

        for po in repo.whatProvides (r, f, v):
            if not po.name in requires:
                requires.append (po.name)

    return requires

def _iter_pkgs (repo, req_prog_cb, create_pkg_cb, user_data):
    pkgs = repo.pkgSack.returnPackages ()

    pkg_names = {}

    n_pkgs = len (pkgs)
    i = 0

    for po in pkgs:
        if not pkg_names.has_key (po.name):
            pkg_names[po.name] = ([], [])

        req_prog_cb (i, n_pkgs - 1, po.name)

        for req in _get_requires (repo, po):
            if not req == po.name and not req in pkg_names[po.name][0]:
                pkg_names[po.name][0].append (req)

        i += 1

    for pkg_name in pkg_names.keys ():
        for req in pkg_names[pkg_name][0]:
            if pkg_names.has_key (req) and \
               not pkg_name in pkg_names[req][1]:
                pkg_names[req][1].append (pkg_name)

    for pkg_name in pkg_names.keys ():
        create_pkg_cb (pkg_name,
                       pkg_names[pkg_name][0],
                       pkg_names[pkg_name][1],
                       user_data)

def _poke_repo (config_path, req_prog_cb, create_pkg_cb, user_data):
    repo = yum.YumBase ()
        
    repo.doConfigSetup (fn = config_path, init_plugins = False)

    cache_dir = tempfile.mkdtemp (prefix = "repoke-cache-")

    try:
        repo.repos.setCacheDir (cache_dir)

        for r in repo.repos.repos.values ():
            r.disable ()

        repo.repos.getRepo ("repoke_tmp_repo").enable ()

        repo.doRepoSetup ()
        repo.doSackSetup ()

        _iter_pkgs (repo, req_prog_cb, create_pkg_cb, user_data)
    finally:
        shutil.rmtree (cache_dir, ignore_errors = True)

def repoke (repo_url, req_prog_cb, create_pkg_cb, user_data):
    (fd, config_path) = tempfile.mkstemp (prefix = "repoke-config-")

    try:
        _write_config (fd, repo_url)

        _poke_repo (config_path, req_prog_cb, create_pkg_cb, user_data)
    finally:
        os.remove (config_path)
